PATH = "path"
SRC_PATH = "src_path"
CONFIG_PATH = "config_path"
USERNAME = "username"
EMAIL = "email"

NOTES_PATH = "notes_path"
NOTES_INDEX_PATH = "notes_index_path"
NOTES_INDEX_FILENAME = "idx_notes.json"

CONFIG_FILENAME = "config.json"
NOTES_DIRNAME = "notes"

INITIALIZED_FLAG = "initialized"
IDX_INITIALIZED_FLAG = "idx_initialized"
